﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;

namespace Musiclibrary
{
	/// <summary>
	/// Interaction logic for FindLibraries.xaml
	/// </summary>
	public partial class FindLibraries : Popup
	{
		public FindLibraries()
		{
			this.InitializeComponent();

            this.Discover();
		}
		

        /// <summary>
        /// Discover remote libraries asynchronously.
        /// </summary>
		private void Discover() {
			this.Refresh.IsEnabled = false;
            Mouse.OverrideCursor = Cursors.Wait;

			var thread = new Thread(new ThreadStart(() => {
                var libraries = RemoteLibrary.Discover();
                this.Dispatcher.Invoke(new Action(() => {
                    this.Libraries.ItemsSource = libraries;
					this.Refresh.IsEnabled = true;
                    Mouse.OverrideCursor = null;
				}), new object[0]);
            }));
            thread.Start();	
		}

		private void Connect_Click(object sender, System.Windows.RoutedEventArgs e)
		{
            var item = this.Libraries.GetItemFromEventSender<RemoteLibraryInfo>(sender);

            if (item != null) {
                try {
                    var library = item.Connect();

                    App.This.RemoteLibraries.Add(item.EndPoint, library);

                    library.Disconnected += new EventHandler((sender2, e2) => 
                        this.Dispatcher.Invoke(new Action(() => {
                            MainWindow.Instance.LoadPlaylists();
                            for (var i = 0; i < App.This.Player.Playlist.Count; ++i) {
                                var song = App.This.Player.Playlist[i] as RemoteSong;
                                if (song != null && song.Library == library)
                                    App.This.Player.Playlist.RemoveAt(i--);
                            }
                        }), new object[0]));

                    MainWindow.Instance.LoadPlaylists();

                    //Refresh list: Now the disconnect button should be shown
					this.Libraries.Items.Refresh();
                } catch (Exception ex) {
                    MessageBox.Show("Ooops! The connection failed. If you are a pro, the following information may help you. Otherwise, just click OK.\r\n" + ex.Message, "Musiclibrary", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
		}

		private void Refresh_Click(object sender, System.Windows.RoutedEventArgs e)
		{
			this.Discover();
		}

		private void Close_Click(object sender, System.Windows.RoutedEventArgs e)
		{
			Mouse.OverrideCursor = null;
			this.Adorner.Hide();
		}

		private void Disconnect_Click(object sender, System.Windows.RoutedEventArgs e)
		{
            var item = this.Libraries.GetItemFromEventSender<RemoteLibraryInfo>(sender);

            if (item != null) {
                var library = App.This.RemoteLibraries[item.EndPoint];
                App.This.RemoteLibraries.Remove(item.EndPoint);

                library.Disconnect();

                this.Libraries.Items.Refresh();
            }
		}


	}
}