﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Timers;

namespace Musiclibrary {
    /// <summary>
    /// A popup (not the kind using DialogAdorner) shown in the bottom left corner of the screen
    /// when the current song changes and the window is not opened.
    /// It's called Toast because the popup jumps out of the Taskbar like a toast out of the toaster.
    /// </summary>
    public partial class Toast : Window {
        private Toast() {
            InitializeComponent(); 
        }

        private static Toast _instance;

        public static Toast Instance
        {
            get
            {
                if (Toast._instance == null)
                    Toast._instance = new Toast();

                return Toast._instance;
            }
        }

        private Timer HideTimer;

        /// <summary>
        /// Let the toast jump out!
        /// </summary>
        public void Show(ISong song) {
            this.SongTitle.Text = song.Title;
            this.Artist.Text = song.Artist;
            this.Album.Text = song.Album;

            //If toast is already flying, don't let it jump out again
            if (this.HideTimer != null) this.HideTimer.Stop();

            this.HideTimer = new Timer(2000);
            this.HideTimer.Elapsed += new ElapsedEventHandler(HideTimer_Elapsed);
            this.HideTimer.Start();

            if (this.Visibility != Visibility.Visible)
            {
                var screen = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea;

                var storyboard = new Storyboard();

                var slideIn = new DoubleAnimation(screen.Height, screen.Height - this.Height, new TimeSpan(0, 0, 0, 0, 300));
                slideIn.DecelerationRatio = 0.5;
                storyboard.Children.Add(slideIn);
                Storyboard.SetTarget(slideIn, this);
                Storyboard.SetTargetProperty(slideIn, new PropertyPath("Top"));

                this.Opacity = 0;
                base.Show();

                this.Left = screen.Width - this.Width;
                storyboard.Begin();
                this.Opacity = 1;
            }
        }

        void HideTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            this.Dispatcher.Invoke(new Action(() =>
                this.Hide()), new object[0]);
        }

        /// <summary>
        /// The toast falls back into the toaster. Of course this is not realistic. 
        /// If it would be realistic, the toast would fall to the side and lie on the taskbar
        /// like the toast on the table.
        /// </summary>
        public new void Hide() {
            var screen = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea;

            var storyboard = new Storyboard();
            var slideOut = new DoubleAnimation(screen.Height, new TimeSpan(0, 0, 0, 0, 300));
            slideOut.AccelerationRatio = 0.5;
            storyboard.Children.Add(slideOut);
            Storyboard.SetTarget(slideOut, this);
            Storyboard.SetTargetProperty(slideOut, new PropertyPath("Top"));

            storyboard.Begin();

            new TimeSpan(0, 0, 0, 0, 300).After(() => this.Close(), this);
        }
    }
}
