﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace Musiclibrary {
    /// <summary>
    /// Represents a playlist.
    /// This interface is, like ILibrary, necessary because there are local and remote playlists.
    /// But there is another use for it: In the ListBox at the left, not only all playlists are shown,
    /// but also items like "All songs" and "Current playlist". These items are also represented by
    /// objects implementing IPlaylist.
    /// </summary>
    public interface IPlaylist {
        /// <summary>
        /// Returns the playlist's name.
        /// </summary>
        string Name { get; }

        /// <summary>
        /// Returns all of the playlist's songs.
        /// </summary>
        IEnumerable<ISong> Songs { get; }

        /// <summary>
        /// The library this playlist belongs to.
        /// This property is needed for grouping in the ListBox at the left.
        /// </summary>
        ILibrary Library { get; }

        /// <summary>
        /// Determines whether the playlist itself is editable (which currently only means,
        /// that it can be Delete()d).
        /// </summary>
        bool IsEditable { get; }

        /// <summary>
        /// Deletes the whole playlist.
        /// </summary>
        void Delete();

        /// <summary>
        /// Determines whether the items of the playlist can be edited 
        /// via the AddSong and DeleteSong functions.
        /// </summary>
        bool ItemsEditable { get; }

        /// <summary>
        /// Adds a song to the end of the playlist.
        /// </summary>
        /// <param name="song"></param>
        void AddSong(ISong song);

        /// <summary>
        /// Deletes a certain song from the playlist.
        /// </summary>
        /// <param name="song"></param>
        void DeleteSong(ISong song);
    }
}
