﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;

namespace Musiclibrary {
    public class PlaylistSource : ISongSource {
        protected Playlist Playlist { get; set; }

        public PlaylistSource(Playlist playlist) {
            this.Playlist = playlist;
        }

        #region ISongSource Member

        public virtual ILibrary Library
        {
            get
            {
                return MainWindow.Instance.Library;
            }
        }

        public virtual string Name {
            get {
                return this.Playlist.Name;
            }
            set {
                this.Playlist.Name = value;
                MainWindow.Instance.Library.SubmitChanges();
            }
        }

        public virtual void Delete() {
            MainWindow.Instance.Library.Playlists.DeleteOnSubmit(this.Playlist);
            MainWindow.Instance.Library.SubmitChanges();
        }

        public virtual bool IsEditable {
            get { return true; }
        }

        public virtual void AddSong(ISong song) {
            if (!(song is Song)) throw new NotSupportedException("Can only add local songs");

            var item = new Playlistitem();
            item.PlaylistPlaylist = this.Playlist;
            item.SongSong = song as Song;
            MainWindow.Instance.Library.Playlistitems.InsertOnSubmit(item);
            MainWindow.Instance.Library.SubmitChanges();
        }

        public virtual void DeleteSong(ISong song) {
            if (!(song is Song)) return;

            var items = from i in this.Playlist.Playlistitems where i.SongSong == song as Song select i;
            foreach (var item in items) {
                MainWindow.Instance.Library.Playlistitems.DeleteOnSubmit(item);
                MainWindow.Instance.Library.SubmitChanges();
                return;
            }
        }

        public virtual bool CanEditSongs {
            get { return true; }
        }

        public IEnumerable<ISong> GetSongs() {
            return from i in this.Playlist.Playlistitems select i.SongSong as ISong;
        }

        public ContextMenu ContextMenu
        {
            get
            {
                var delete = new MenuItem() { Header = "Delete" };
                delete.Click += new System.Windows.RoutedEventHandler((sender, e) =>
                {
                    this.Delete();
                });

                var menu = new ContextMenu();
                return menu;
            }
        }

        #endregion
    }
}
