﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Musiclibrary
{
    /// <summary>
    /// Represents a song from the local library
    /// </summary>
    public partial class Song : ISong
    {
        public string GetPath(string dir, string schema, bool createNewFile)
        {
            var path = System.IO.Path.Combine(dir,
                schema.Replace("{Title}", this.Title.RemoveInvalidPathChars())
                .Replace("{Artist}", this.Artist.RemoveInvalidPathChars())
                .Replace("{Album}", this.Album.RemoveInvalidPathChars())
                .Replace("{Year}", this.Year.ToString().RemoveInvalidPathChars())
                .Replace("{Genre}", this.Genre.RemoveInvalidPathChars()));

            var extension = System.IO.Path.GetExtension(this.Path);
            if (!extension.StartsWith("."))
                extension = "." + extension;

            if (createNewFile && File.Exists(path + extension))
            {
                var i = 2;
                while (File.Exists(path + " (" + i.ToString() + ")" + extension))
                    ++i;

                path += " (" + i.ToString() + ")";
            }

            path += extension;

            return path;
        }

        /// <summary>
        /// Updates the metadata in the song file
        /// </summary>
        public void UpdateMetadata()
        {
            var info = MetadataBase.LoadMetadata(this.Path);

            info.Title = this.Title;
            info.Artist = this.Artist;
            info.Album = this.Album;
            info.Genre = this.Genre;
            info.Year = this.Year;

            info.Save();
        }


        public ILibrary Library { get; set; }

        /// <summary>
        /// Returns the data from the song file
        /// </summary>
        /// <returns></returns>
        public byte[] GetData() {
            var stream = new FileInfo(this.Path).OpenRead();
            var data = new byte[stream.Length];
            stream.Read(data, 0, (int)stream.Length);
            stream.Close();

            return data;
        }

        #region ISong Member

        public string Load() {
            return this.Path;
        }

        #endregion
    }
}
