﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HundredMilesSoftware.UltraID3Lib;

namespace Musiclibrary {
    /// <summary>
    /// Provides an implementation of MetadataBase using the UltraID3 library.
    /// </summary>
    public class UltraID3Metadata :  MetadataBase {
        protected UltraID3 Info;

        public UltraID3Metadata() {
            this.Info = new UltraID3();
        }


        // Provide access to the metadata
        // Make sure both ID3v1 and ID3v2 are supported
        // ID3v1 supports only 30-character-long artist, title and album
        // and only predefined genres

        private string _path;
        public override string Path { get { return this._path; } }

        public override string Title {
            get {
                return this.Info.Title;
            }
            set {
                this.Info.ID3v2Tag.Title = value;
                if (value.Length > 30)
                    value = value.Substring(0, 30);
                this.Info.ID3v1Tag.Title = value;
            }
        }

        public override string Artist {
            get {
                return this.Info.Artist;
            }
            set {
                this.Info.ID3v2Tag.Artist = value;
                if (value.Length > 30)
                    value = value.Substring(0, 30);
                this.Info.ID3v1Tag.Artist = value;
            }
        }

        public override string Album {
            get {
                return this.Info.Album;
            }
            set {
                this.Info.ID3v2Tag.Album = value;
                if (value.Length > 30)
                    value = value.Substring(0, 30);
                this.Info.ID3v1Tag.Album = value;
            }
        }

        public override string Genre {
            get {
                return this.Info.Genre;
            }
            set {
                this.Info.ID3v2Tag.Genre = value;
                
                try {
                    this.Info.ID3v1Tag.SetGenre(value);
                } catch { }
            }
        }

        public override int? Year {
            get {
                return (int?)this.Info.Year;
            }
            set {
                this.Info.ID3v2Tag.Year = (short?)value;
                if (!value.HasValue || value.Value < 10000)
                    this.Info.ID3v1Tag.Year = (short?)value;
            }
        }

        public override void Save() {
            this.Info.Write();
        }

        public void Load(string file) {
            this.Info.Read(file);
            if (this.Info.GetExceptions().Where(x => x.ExceptionLevel == ID3ExceptionLevels.Error).Count() > 0)
                throw new Exception("The metadata could not be read.");
            this._path = file;
        }
    }
}
