﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
namespace Musiclibrary
{
    /// <summary>
    /// Watches the observed directories and automatically adds and remove the files to the library.
    /// </summary>
    public class DirectoryWatcher
    {
        public DirectoryWatcher() {
            this.Watchers = new List<FileSystemWatcher>();
        }

        protected List<FileSystemWatcher> Watchers { get; private set; }

        public void LoadDirectories()
        {
            foreach (var watcher in this.Watchers)
            {
                watcher.Created -= this.watcher_Created;
                watcher.Deleted -= this.watcher_Deleted;
                watcher.Renamed -= this.watcher_Renamed;
            }

            this.Watchers.Clear();

            var importer = new Importer(MainWindow.Instance);

            foreach (var dir in App.This.Library.MonitoredDirs)
            {
                var watcher = new FileSystemWatcher(dir.Path, "*.mp3");
                watcher.Created += new FileSystemEventHandler(watcher_Created);
                watcher.Deleted += new FileSystemEventHandler(watcher_Deleted);
                watcher.Renamed += new RenamedEventHandler(watcher_Renamed);
                watcher.EnableRaisingEvents = true;
                watcher.IncludeSubdirectories = true;
                this.Watchers.Add(watcher);

                importer.FilesAndDirs.Add(new DirectoryInfo(dir.Path));
            }

            importer.Begin();
        }

        void watcher_Renamed(object sender, RenamedEventArgs e)
        {
            var song = from i in App.This.Library.Songs where i.Path == e.OldFullPath select i;
            if (song.Count() > 0) song.Single().Path = e.FullPath;
            App.This.Library.SubmitChanges();
            MainWindow.Instance.Dispatcher.Invoke(new Action(() =>
                MainWindow.Instance.LoadSongs()), new object[0]);
        }

        void watcher_Deleted(object sender, FileSystemEventArgs e)
        {
            var song = from i in App.This.Library.Songs where i.Path == e.FullPath select i;
            if (song.Count() > 0) App.This.Library.Songs.DeleteOnSubmit(song.Single());

            App.This.Library.SubmitChanges();

            MainWindow.Instance.Dispatcher.Invoke(new Action(() =>
                MainWindow.Instance.LoadSongs()), new object[0]);
        }

        void watcher_Created(object sender, FileSystemEventArgs e)
        {
            var importer = new Importer(MainWindow.Instance);
            importer.FilesAndDirs.Add(new FileInfo(e.FullPath));
            importer.Begin();

            importer.Done += new EventHandler<WorkerCompletedEventArgs>((sender2, e2) =>
                MainWindow.Instance.Dispatcher.Invoke(new Action(() =>
                    MainWindow.Instance.LoadSongs()), new object[0]));
        }


    }
}
