﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Windows;
using System.Collections.ObjectModel;
using System.IO;
using System.Net;
using System.Data.SqlServerCe;

namespace Musiclibrary {
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application {
        public App() {
            this.Player = new WMPPlayer();
            this.RemoteLibraries = new Dictionary<EndPoint, RemoteLibrary>();
            this.SharingServer = new SharingServer();
            this.Watcher = new DirectoryWatcher();
            this.Keylogger = new Keylogger();
            base.Exit += new ExitEventHandler(App_Exit);
        }


        void App_Exit(object sender, ExitEventArgs e) {
            this.SharingServer.Stop();
            this.Keylogger.Stop();
        }

        public IPlayer Player { get; protected set; }
        public Library Library { get; set; }
        public Dictionary<EndPoint, RemoteLibrary> RemoteLibraries { get; private set; }
        public SharingServer SharingServer { get; protected set; }
        public DirectoryWatcher Watcher { get; protected set; }
        public Keylogger Keylogger { get; protected set; }

        public void Load() {
            if (!File.Exists(Musiclibrary.Properties.Settings.Default.MediaLibrary)) {
                if (File.Exists(this.DefaultDatabase))
                    this.LoadLibrary(this.DefaultDatabase);
            } else
                this.LoadLibrary(Musiclibrary.Properties.Settings.Default.MediaLibrary);

            this.Keylogger.Start();
        }

        public static App This {
            get { return App.Current as App; }
        }

        /// <summary>
        /// Returns the default location for the music database, which is in the folder My Music/Musiclibrary.
        /// </summary>
        public string DefaultDatabase {
            get {
                var dbPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyMusic), "Musiclibrary");
                dbPath = Path.Combine(dbPath, "Library.sdf");
                return dbPath;
            }
        }

        /// <summary>
        /// Creates a new library at the default location received from DefaultDatabase property.
        /// </summary>
        public void CreateLibrary() {
            var path = this.DefaultDatabase;

            if (File.Exists(path))
                File.Delete(path);

            var dir = Path.GetDirectoryName(path);

            if (!Directory.Exists(dir))
                Directory.CreateDirectory(dir);

            Library.Create(path);
            this.LoadLibrary(path);

            this.Library.SubmitChanges();

            //Choose organization in My Music\Musiclibrary by default
            App.This.Library.Organize = true;
            App.This.Library.Location = Path.GetDirectoryName(App.This.DefaultDatabase);
            App.This.Library.Schema = "{Artist}\\{Album}\\{Title}";
            App.This.Library.SubmitChanges();

            Musiclibrary.Properties.Settings.Default.MediaLibrary = path;
            Musiclibrary.Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Loads the specified music library.
        /// </summary>
        public void LoadLibrary(string path) {
            if (this.Library != null)
                this.Library.Dispose();

            //Test if able to connect to database
            //Strange to say, when creating the DataContext directly, no Exception is thrown
            //if the file is not a valid database, but the application hangs.
            var conn = new SqlCeConnection("Data Source=" + path);
            conn.Open();
            conn.Close();

            var db = new Library("Data Source=" + path);
           
            this.Library = db;

            this.Watcher.LoadDirectories();
        }
    }

}
